-- myDebug.e
-- v1.1
-- 2012-09-09
-- debug controls
-- adapted to Euphoria 4.1

include std/graphics.e
include std/io.e
include std/sequence.e
include std/types.e
include std/text.e
include std/wildcard.e
include std/win32/msgbox.e
include myLibs/myTypes.e

global integer  f_debug = 1    -- debug file
global integer  debugLevel = 0   -- 0=no debug, 1=debug file only, 2=debug_file and screen output

------------------------------------------------------------------------------

public function dateStamp(sequence msg)
-- prefixes the message by date and time for logging
  sequence cur_date

  cur_date = date()
  return sprintf("%d-%02d-%02d %02d:%02d:%02d -> %s\n", {
                             (cur_date[1] + 1900), cur_date[2], cur_date[3],
                              cur_date[4], cur_date[5], cur_date[6], msg
                            })
end function

------------------------------------------------------------------------------

public procedure logMsg(sequence msg)
-- records logs
  if debugLevel>0 then
    puts(f_debug, dateStamp(msg))
    flush(f_debug)
  end if
  if debugLevel>1 then
    puts(1, dateStamp(msg))
  end if
end procedure

------------------------------------------------------------------------------

procedure displayError(sequence msg)
-- prints a message in a frame at position pos
-- if pos is an atom (mostly 0), then center the frame in the screen
  void = message_box(msg, "Erreur", {MB_ICONERROR+MB_OK})
end procedure

------------------------------------------------------------------------------

public procedure warnError(sequence msg, atom quit)
-- logs an error and displays a warning
-- if quit is set then abort
  logMsg(msg)
  displayError(msg)
  if quit then
    abort(1)
  end if
end procedure

------------------------------------------------------------------------------

public function showPrintable(sequence s)
-- returns only printable characters of a sequence
-- non printable characters are replace by a dot
  sequence res

  res = ""
  for i = 1 to length(s) do
    if integer(s[i]) then
      if (s[i] > 31) and (s[i] < 255) then
        res &= s[i]
      elsif s[i] = 9 then
        res &= "\\t"
      elsif s[i] = 13 then
        res &= "\\r"
      elsif s[i] = 10 then
        res &= "\\n"
      else
        res &= "."
      end if
    else
      res &= "."
    end if
  end for
  return res
end function

------------------------------------------------------------------------------

public function sequenceDump(sequence s, integer detailed=0)
-- prints a sequence structure in a human readable way as one string
  integer subSequence
  sequence result = ""

  if isString(s) then
    result = sprintf("\"%s\"", {showPrintable(s)})
  else
    subSequence = 0
    for i=1 to length(s) do
      if sequence(s[i]) then
        subSequence = 1
        exit
      end if
    end for
    if subSequence then
      result = sequenceDump(s[1])
      for i=2 to length(s) do
        result &= ", " & sequenceDump(s[i])
      end for
      result = "{" & result & "}"
    else
      result = sprint(s) & "\t'" & showPrintable(s) & "'"
    end if
  end if
  return result
end function

------------------------------------------------------------------------------

public procedure objectDump(object x, sequence variable, integer output=1, integer detailed=0)
-- prints an object in a human readable way
  integer subSequence


--puts(output, variable&" = "&sprint(x)&"\n")
  if integer(x) then
    printf(output, "%s = %d\n", {variable, x})
  elsif atom(x) then
    if (x >= 0) and (x = floor(x)) then
      printf(output, "%s = %.0f\n", {variable, x})
    elsif (x < 0) and (x = floor(x+1)) then
      printf(output, "%s = %.0f\n", {variable, x})
    else
      printf(output, "%s = %f\n", {variable, x})
    end if
  elsif isString(x) then
    if detailed then
      printf(output, "%s = \"%s\"\t%s\n", {variable, x, sprint(x)})
    else
      printf(output, "%s = \"%s\"\n", {variable, x})
    end if
  else
    subSequence = 0
    for i=1 to length(x) do
      if sequence(x[i]) then
        subSequence = 1
        exit
      end if
    end for
    if subSequence then
      for i=1 to length(x) do
        objectDump(x[i], variable&sprintf("[%d]", i), output)
      end for
    else
      if detailed then
        printf(output, "%s = %s\t\"%s\"\n", {variable, sprint(x), showPrintable(x)})
      else
        printf(output, "%s = %s\n", {variable, sprint(x)})
      end if
    end if
  end if
  if output=f_debug then flush(f_debug) end if
end procedure

------------------------------------------------------------------------------

function translateVariable(sequence translator, sequence path)
  integer n
  
  n = path[$]
  for i = 1 to length(translator) do
    if length(translator[i]) != 2 then
      warnError("Each translation line must have 2 sequences: path & parameter values!", 0)
      return sprintf("%d",n)
    end if
    if is_match(translator[i][1], path[1..$-1]) then
      if n <= length(translator[i][2]) then
        return translator[i][2][n]
      else
        return sprintf("%d",n)
      end if
    end if
  end for
  return sprintf("%d",n)
end function

------------------------------------------------------------------------------

procedure recurseSequence(object x, sequence Name, integer level,
                          sequence path, integer Output, integer detailed,
                          sequence translator)
-- prints a sequence structure in a human readable way
  integer subSequence
  sequence s, offset, prefix

  offset = ""
  prefix = ""
  if level > 0 then
    for i = 1 to level do offset &= ".  " end for
    if length(translator) > 0 then
      prefix = offset & "[" & translateVariable(translator, path) & "]"
    else
      prefix = offset & sprintf("[%d]", path[$])
    end if
  end if
  s = ""
  if Output = 0 then Output = 1 end if
  if integer(x) then
    s = sprintf("%s = %d", {Name, x})
    puts(Output, prefix&s&"\n")
  elsif atom(x) then
    if (x >= 0) and (x = floor(x)) then
      s = sprintf("%s = %.0f", {Name, x})
    elsif (x < 0) and (x = floor(x+1)) then
      s = sprintf("%s = %.0f", {Name, x})
    else
      s = sprintf("%s = %f", {Name, x})
    end if
    puts(Output, prefix&s&"\n")
  elsif isString(x) then
    if detailed then
      s = sprintf("%s = \"%s\"\t%s", {Name, x, sprint(x)})
    else
      s = sprintf("%s = \"%s\"", {Name, x})
    end if
    puts(Output, prefix&s&"\n")
  else
    subSequence = 0
    for i=1 to length(x) do
      if sequence(x[i]) then
        subSequence = 1
        exit
      end if
    end for
    if subSequence = 0 then
      if detailed then
        s = sprintf("%s = %s\t\"%s\"", {Name, sprint(x), showPrintable(x)})
      else
        s = sprintf("%s = %s", {Name, sprint(x)})
      end if
      puts(Output, prefix&s&"\n")
    else
      if length(s) = 0 then
        puts(Output, prefix&"\n")
      end if
      for i=1 to length(x) do
        recurseSequence(x[i], Name, level+1, append(path,i), Output, detailed, translator)
      end for
    end if
  end if
  if Output=f_debug then flush(f_debug) end if
end procedure

------------------------------------------------------------------------------

public procedure analyzeSequence(object x, sequence Name, integer Output=1, integer detailed=0, sequence translator= {})
  printf(Output, "%s", {Name})
  recurseSequence(x, "", 0, {}, Output, detailed, translator)
end procedure
